# -*- coding: utf-8 -*-
# Copyright (C) 2007-2010 Samalyse SARL

# This file is part of Telemeta.

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Authors: Olivier Guilyardi <olivier@samalyse.com>
#          David LIPSZYC <davidlipszyc@gmail.com>

from telemeta.models.core import *
from django.utils.translation import ugettext_lazy as _


class Enumeration(ModelCore):
    "Abstract enumerations base class"

    hidden = False
    value = CharField(_('value'), required=True, unique=True)
    notes = TextField(_('notes'))

    def __unicode__(self):
        return self.value

    class Meta(MetaCore):
        abstract = True

class MetaEnumeration(MetaCore):
    ordering = ['value']

class PhysicalFormat(Enumeration):
    "Collection physical format"

    class Meta(MetaEnumeration):
        db_table = 'physical_formats'
        verbose_name = _("archive format")

class PublishingStatus(Enumeration):
    "Collection publishing status"

    class Meta(MetaEnumeration):
        db_table = 'publishing_status'
        verbose_name = _("secondary edition")

class AcquisitionMode(Enumeration):
    "Mode of acquisition of the collection"

    class Meta(MetaEnumeration):
        db_table = 'acquisition_modes'
        verbose_name = _("mode of acquisition")

class MetadataAuthor(Enumeration):
    "Collection metadata author"

    class Meta(MetaEnumeration):
        db_table = 'metadata_authors'
        verbose_name = _("record author")

class MetadataWriter(Enumeration):
    "Collection metadata writer"

    class Meta(MetaEnumeration):
        db_table = 'metadata_writers'
        verbose_name = _("record writer")

class LegalRight(Enumeration):
    "Collection legal rights"

    class Meta(MetaEnumeration):
        db_table = 'legal_rights'
        verbose_name = _("legal rights")

class RecordingContext(Enumeration):
    "Collection recording context"

    class Meta(MetaEnumeration):
        db_table = 'recording_contexts'
        verbose_name = _("recording context")

class AdConversion(Enumeration):
    "Collection digital to analog conversion status"

    class Meta(MetaEnumeration):
        db_table = 'ad_conversions'
        verbose_name = _("A/D conversion")

class VernacularStyle(Enumeration):
    "Item vernacular style"

    class Meta(MetaEnumeration):
        db_table = 'vernacular_styles'
        verbose_name = _("vernacular style")

class GenericStyle(Enumeration):
    "Item generic style"

    class Meta(MetaEnumeration):
        db_table = 'generic_styles'
        verbose_name = _("generic style")

class ContextKeyword(Enumeration):
    "Keyword"

    class Meta(MetaEnumeration):
        db_table = 'context_keywords'
        verbose_name = _("keyword")

class Publisher(Enumeration):
    "Collection publisher"

    class Meta(MetaEnumeration):
        db_table = 'publishers'
        verbose_name = _("publisher")


class PublisherCollection(ModelCore):
    "Collection which belongs to publisher"

    publisher = ForeignKey('Publisher', related_name="publisher_collections", verbose_name=_('publisher'))
    value     = CharField(_('value'), required=True)

    def __unicode__(self):
        return self.value

    class Meta(MetaCore):
        db_table = 'publisher_collections'
        ordering = ['value']


class EthnicGroup(Enumeration):
    "Item ethnic group"

    class Meta(MetaEnumeration):
        db_table = 'ethnic_groups'
        verbose_name = _('population / social group')


class EthnicGroupAlias(ModelCore):
    "Item ethnic group other name"

    ethnic_group = ForeignKey('EthnicGroup', related_name="aliases", verbose_name=_('population / social group'))
    value        = CharField(_('name'), required=True)

    class Meta(MetaCore):
        db_table = 'ethnic_group_aliases'
        unique_together = (('ethnic_group', 'value'),)
        ordering = ['ethnic_group__value']


# Tape formats
class TapeWheelDiameter(Enumeration):
    "Tape wheel diameter (cm)"

    hidden = True

    class Meta(MetaEnumeration):
        db_table = 'tape_wheel_diameter'
        verbose_name = _("tape wheel diameter (cm)")

class TapeLength(Enumeration):
    "Tape length (cm)"

    hidden = True

    class Meta(MetaEnumeration):
        db_table = 'tape_length'
        verbose_name = _("tape length (cm)")

class TapeWidth(Enumeration):
    "Tape width (inch)"

    hidden = True

    class Meta(MetaEnumeration):
        db_table = 'tape_width'
        verbose_name = _("tape width (inch)")

class TapeSpeed(Enumeration):
    "Tape speed (cm/s)"

    hidden = True

    class Meta(MetaEnumeration):
        db_table = 'tape_speed'
        verbose_name = _("tape speed (cm/s)")

class TapeVendor(Enumeration):
    "Tape vendor"

    hidden = True

    class Meta(MetaEnumeration):
        db_table = 'tape_vendor'
        verbose_name = _("tape brand")


class NumberOfChannels(Enumeration):
    "Number of channels"

    hidden = True

    class Meta(MetaEnumeration):
        db_table = 'original_channel_number'
        verbose_name = _("number of channels")


class Organization(Enumeration):
    "Organization"

    hidden = True

    class Meta(MetaEnumeration):
        db_table = 'organization'
        verbose_name = _("organization")


class Rights(Enumeration):
    "Archive rights"

    hidden = True

    class Meta(MetaEnumeration):
        db_table = 'rights'
        verbose_name = _("rights")


class Topic(Enumeration):
    "Topic, subject of the study, research, etc.."

    hidden = True

    class Meta(MetaEnumeration):
        db_table = 'topic'
        verbose_name = _("topic")


class CopyType(Enumeration):
    "Type of the copy"

    class Meta(MetaEnumeration):
        db_table = 'copy_type'
        verbose_name = _("copy type")


class MediaType(Enumeration):
    "Type of the media"

    class Meta(MetaEnumeration):
        db_table = 'media_type'
        verbose_name = _("media type")


class OriginalFormat(Enumeration):
    "Original format"

    class Meta(MetaEnumeration):
        db_table = 'original_format'
        verbose_name = _("original format")


class Status(Enumeration):
    "Resource status"

    class Meta(MetaEnumeration):
        db_table = 'media_status'
        verbose_name = _("collection status")


class IdentifierType(Enumeration):
    "Identifier type"

    class Meta(MetaEnumeration):
        db_table = 'identifier_type'
        verbose_name = _("identifier type")




from optparse import make_option
from django.conf import settings
from django.core.management.base import BaseCommand, CommandError
from django.core.files.base import ContentFile
from django.contrib.auth.models import User
from telemeta.models import *
from telemeta.util.unaccent import unaccent
import os, re, glob

try:
    from django.utils.text import slugify
except ImportError:
    def slugify(string):
        killed_chars = re.sub('[\(\),]', '', string)
        return re.sub(' ', '_', killed_chars)

def beautify(string):
    return os.path.splitext(string)[0].replace('_',' ')

def cleanup_dir(root_dir):
    for resource in os.listdir(root_dir):
        path = os.path.join(root_dir, resource)
        if os.path.isdir(path):
            new_path = path.replace(' ', '_')
            new_path = new_path.replace('son_', '')
            new_path = new_path.replace('son', '')
            if new_path != path:
                os.rename(path, new_path)
            cleanup_dir(new_path)

def trim_list(list):
    new = []
    for item in list:
        if item:
            new.append(item)
    return new

def reset():
    for i in MediaItem.objects.all():
        i.delete()
    for c in MediaCollection.objects.all():
        c.delete()


class Command(BaseCommand):
    help = "import media files from a directory to a corpus"
    args = "root_dir"
    media_formats = ['mp3']
    image_formats = ['png', 'jpg']
    text_formats = ['txt']
    media_root = settings.MEDIA_ROOT
    dry_run = False
    user = User.objects.get(username='admin')

    def write_file(self, item, media):
        filename = media.split(os.sep)[-1]
        print media
        if os.path.exists(media):
            if not item.file or self.force:
                if not self.media_root in self.source_dir:
                    print "file not in MEDIA_ROOT, copying..."
                    f = open(media, 'r')
                    if not self.dry_run:
                        file_content = ContentFile(f.read())
                        item.file.save(filename, file_content)
                        item.save()
                    f.close()
                else:
                    print "file in MEDIA_ROOT, linking..."
                    path = media.replace(self.media_root, '')
                    if not self.dry_run:
                        item.file = path
                        item.save()
                if self.user:
                    item.set_revision(self.user)

    def handle(self, *args, **options):
        # NOT4PROD!!
        reset()

        root_dir = args[-1]
        self.source_dir = root_dir
        print self.source_dir
        print self.media_root
        cleanup_dir(self.source_dir)
        chapters = os.listdir(self.source_dir)
        corpus_name = os.path.split(root_dir)[-1]
        corpus_id = slugify(unicode(corpus_name))

        cc = MediaCorpus.objects.filter(code=corpus_id)
        if cc:
            corpus = cc[0]
        else:
            corpus = MediaCorpus(code=corpus_id)
            corpus.title = corpus_name
            corpus.save()

        for chapter in chapters:
            chapter_dir = os.path.join(self.source_dir, chapter)
            metadata = {}

            for filename in os.listdir(chapter_dir):
                path = os.path.join(chapter_dir, filename)
                if os.path.isfile(path) and '.txt' == os.path.splitext(filename)[1]:
                    f = open(path, 'r')
                    i = 0
                    for line in f.readlines():
                        data = re.split(r'\t+', line.rstrip('\t'))
                        if i == 0:
                            chapter_title = data[1]
                            print chapter_title
                        else:
                            metadata[data[0]] = data[1:]
                        i += 1
                    print metadata

            collection_name = chapter
            collection_id = corpus_id + '_' + slugify(unicode(collection_name))
            collection_title = collection_name.replace('_', ' ') + ' - ' + chapter_title
            print collection_title
            cc = MediaCollection.objects.filter(code=collection_id, title=collection_title)
            if cc:
                collection = cc[0]
            else:
                collection = MediaCollection(code=collection_id)
                collection.title = collection_title
                collection.save()
            if not collection in corpus.children.all():
                corpus.children.add(collection)

            for filename in os.listdir(chapter_dir):
                path = os.path.join(chapter_dir, filename)
                if os.path.isfile(path) and '.jpg' == os.path.splitext(filename)[1]:
                    related_path = path.replace(self.media_root, '')
                    related, c = MediaCollectionRelated.objects.get_or_create(collection=collection,
                                    file=related_path)

            for root, dirs, files in os.walk(chapter_dir):
                for media_file in files:
                    path = os.path.join(root, media_file)
                    print path
                    new_media_file = slugify(unicode(media_file.decode('utf8')))
                    if new_media_file[-3] != '.':
                        new_media_file = new_media_file[:-3] + '.' + new_media_file[-3:]
                    print new_media_file
                    if new_media_file != media_file:
                        new_media_path = os.path.join(root, new_media_file)
                        os.rename(path, new_media_path)
                        media_file = new_media_file
                        print 'renaming: ' + media_file
                        path = new_media_path

                    media_name = os.path.splitext(media_file)[0]
                    media_ext = os.path.splitext(media_file)[1][1:]

                    if media_ext and media_ext in self.media_formats and media_name[0] != '.':
                        root_list = root.split(os.sep)
                        media_path = os.sep.join(root_list[-4:])  + os.sep + media_file
                        item_name = root_list[-1]
                        item_id = collection_id + '_' + slugify(unicode(item_name))
                        data = metadata[item_name]
                        item, c = MediaItem.objects.get_or_create(collection=collection, code=item_id)
                        item.old_code = item_name
                        self.write_file(item, path)
                        title = data[0].split('.')
                        item.title = title[0].replace('\n', '')
                        print data
                        if len(data) > 1:
                            item.track = data[1].replace('\n', '')
                        if len(title) > 1:
                            item.comment = '. '.join(title[1:])
                        item.save()
                        for related_file in os.listdir(root):
                            related_path = os.sep.join(root_list[-4:]) + os.sep + related_file
                            related_name = os.path.splitext(related_file)[0]
                            related_ext = os.path.splitext(related_file)[1][1:]
                            if related_ext in self.image_formats:
                                related, c = MediaItemRelated.objects.get_or_create(item=item, file=related_path)
                                if len(data) > 2:
                                    related.title = item.track
                                related.set_mime_type()
                                related.save()




#!/usr/bin/env python
# -*- coding: utf-8 -*-
from distutils.core import setup

setup(
    name='saved_searches',
    version='2.0.0-alpha',
    description='Saves user searches for integration with Haystack.',
    author='Daniel Lindsley',
    author_email='daniel@toastdriven.com',
    url='http://github.com/toastdriven/saved_searches',
    packages=[
        'saved_searches',
        'saved_searches.templatetags',
    ],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Utilities'
    ],
)



from django import template
from saved_searches.models import SavedSearch


register = template.Library()


class MostRecentNode(template.Node):
    def __init__(self, varname, user=None, search_key=None, limit=10):
        self.varname = varname
        self.user = user
        self.search_key = search_key
        self.limit = int(limit)
    
    def render(self, context):
        user = None
        search_key = None
        
        if self.user is not None:
            temp_user = template.Variable(self.user)
            user = temp_user.resolve(context)
        
        if self.search_key is not None:
            temp_search_key = template.Variable(self.search_key)
            search_key = temp_search_key.resolve(context)
        
        context[self.varname] = SavedSearch.objects.most_recent(user=user, search_key=search_key)[:self.limit]
        return ''


@register.tag
def most_recent_searches(parser, token):
    """
    Returns the most recent queries seen. By default, returns the top 10.
    
    Usage::
    
        {% most_recent_searches as <varname> [for_user user] [for_search_key search_key] [limit n] %}
    
    Example::
    
        {% most_recent_searches as recent %}
        {% most_recent_searches as recent for_user request.user %}
        {% most_recent_searches as recent for_search_key "general" %}
        {% most_recent_searches as recent limit 5 %}
        {% most_recent_searches as recent for_user request.user for_search_key "general" limit 15 %}
    """
    bits = token.split_contents()
    tagname = bits[0]
    bits = bits[1:]
    
    if len(bits) < 2:
        raise template.TemplateSyntaxError("%r tag requires at least two arguments." % tagname)
    
    varname = bits[1]
    bits = iter(bits[2:])
    user = None
    search_key = None
    limit = 10
    
    for bit in bits:
        if bit == 'for_user':
            user = bits.next()
        if bit == 'for_search_key':
            search_key = bits.next()
        if bit == 'limit':
            limit = bits.next()
    
    return MostRecentNode(varname, user, search_key, limit)


class MostPopularNode(template.Node):
    def __init__(self, varname, user=None, search_key=None, limit=10):
        self.varname = varname
        self.user = user
        self.search_key = search_key
        self.limit = int(limit)
    
    def render(self, context):
        user = None
        search_key = None
        
        if self.user is not None:
            temp_user = template.Variable(self.user)
            user = temp_user.resolve(context)
        
        if self.search_key is not None:
            temp_search_key = template.Variable(self.search_key)
            search_key = temp_search_key.resolve(context)
        
        context[self.varname] = SavedSearch.objects.most_popular(user=user, search_key=search_key)[:self.limit]
        return ''


@register.tag
def most_popular_searches(parser, token):
    """
    Returns the most popular queries seen. By default, returns the top 10.
    
    Usage::
    
        {% most_popular_searches as <varname> [for_user user] [for_search_key search_key] [limit n] %}
    
    Example::
    
        {% most_popular_searches as popular %}
        {% most_popular_searches as popular for_user request.user %}
        {% most_popular_searches as popular for_search_key "general" %}
        {% most_popular_searches as popular limit 5 %}
        {% most_popular_searches as popular for_user request.user for_search_key "general" limit 15 %}
    """
    bits = token.split_contents()
    tagname = bits[0]
    bits = bits[1:]
    
    if len(bits) < 2:
        raise template.TemplateSyntaxError("%r tag requires at least two arguments." % tagname)
    
    varname = bits[1]
    bits = iter(bits[2:])
    user = None
    search_key = None
    limit = 10
    
    for bit in bits:
        if bit == 'for_user':
            user = bits.next()
        if bit == 'for_search_key':
            search_key = bits.next()
        if bit == 'limit':
            limit = bits.next()
    
    return MostPopularNode(varname, user, search_key, limit)



# -*- coding: utf-8 -*-
# Copyright (C) 2011-2012 Parisson SARL

# This file is part of Telemeta.

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Authors:
#          Guillaume Pellerin <yomguy@parisson.com>

from django.utils.translation import ugettext_lazy as _
from django.core.exceptions import ValidationError
from telemeta.models.core import *

SCOPE_CHOICES = (('I', 'Individual'), ('M', 'Macrolanguage'), ('S', 'Special'))

TYPE_CHOICES = (('A', 'Ancient'), ('C', 'Constructed'), ('E', 'Extinct'),
                ('H', 'Historical'), ('L', 'Living'), ('S', 'Special'))

class Language(ModelCore):
    "ISO 639-3 normalized languages"

    identifier      = CharField(_('identifier'), max_length=3)
    part2B          = CharField(_('equivalent ISO 639-2 identifier (bibliographic)'), max_length=3)
    part2T          = CharField(_('equivalent ISO 639-2 identifier (terminologic)'), max_length=3)
    part1           = CharField(_('equivalent ISO 639-1 identifier'), max_length=1)
    scope           = CharField(_('scope'), choices=SCOPE_CHOICES, max_length=1)
    type            = CharField(_('type'), choices=TYPE_CHOICES, max_length=1)
    name            = CharField(_('name'))
    comment         = TextField(_('comment'))

    def __str__(self):
        return self.name.encode('utf-8')

    class Meta(MetaCore):
        db_table = 'languages'
        ordering = ['name']
        verbose_name_plural = _('languages')



#!/usr/bin/python2.5
# -*- coding: utf-8 -*-

import os
import sys

sys.path.append(os.path.dirname('sandbox'))

# os.environ['DJANGO_SETTINGS_MODULE'] = 'settings'
#
# import django.core.handlers.wsgi
# application = django.core.handlers.wsgi.WSGIHandler()

os.environ.setdefault("DJANGO_SETTINGS_MODULE", "sandbox.settings")

from django.core.wsgi import get_wsgi_application
application = get_wsgi_application()



# -*- coding: utf-8 -*-
#
# Copyright (C) 2010 jdunck
# Copyright (C) 2011 Parisson

# This file is part of Telemeta.

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Author: jdunck
# taken from https://github.com/jdunck/python-unicodecsv.git

import csv, codecs, cStringIO

def _stringify(s, encoding):
    if type(s)==unicode:
        return s.encode(encoding)
    elif isinstance(s, (int , float)):
        pass #let csv.QUOTE_NONNUMERIC do its thing.
    elif type(s) != str:
        s=str(s)
    return s

def _stringify_list(l, encoding):
    return [_stringify(s, encoding) for s in l]


class UnicodeCSVWriter(object):
    def __init__(self, f, elements, dialect=csv.excel, encoding="utf-8", **kwds):
        self.dialect = dialect
        self.encoding = encoding
        self.writer = csv.writer(f, dialect=dialect, **kwds)
        self.line = 0
        self.elements = elements
        self.tags = []
        if self.elements:
            self.get_tags(self.elements[0])

    def get_tags(self, element):
        _dict = element.to_dict_with_more()
        for key in _dict.keys():
            if not key in self.tags:
                self.tags.append(key)

        # code and title on the two first column
        self.tags.remove('code')
        self.tags.remove('title')
        self.tags.sort()
        self.tags.insert(0, 'title')
        self.tags.insert(0, 'code')

    def output(self):
        yield self.writer.writerow(self.tags)
        for element in self.elements:
            yield self.writer.writerow(_stringify_list(element.to_row(self.tags), self.encoding))


class Echo(object):
    """An object that implements just the write method of the file-like
    interface.
    """
    def write(self, value):
        """Write the value by returning it, instead of storing in a buffer."""
        return value



from optparse import make_option
from django.conf import settings
from django.core.management.base import BaseCommand, CommandError
from django.core.files.base import ContentFile
from telemeta.models import *
from telemeta.util.unaccent import unaccent
from telemeta.util.url import URLMediaParser
import os


class Command(BaseCommand):
    help = "import media files from a URL directory into a collection with a given prefix"
    args = "collection_code URL"

    def handle(self, *args, **options):
        collection_code = args[0]
        prefix = args[1]
        url = args[2]

        parser = URLMediaParser(url)
        urls = parser.get_urls()
        collections = MediaCollection.objects.filter(code=collection_code)
        if not collections:
            collection = MediaCollection(code=collection_code, title=collection_code)
            collection.public_access = 'full'
            collection.save()
            print 'collection created: ' + collection_code
        else:
            collection = collections[0]
            print 'using collection: ' + collection.code

        for url in urls:
            filename = url.split('/')[-1]
            name, ext = os.path.splitext(filename)
            items = MediaItem.objects.filter(code=name)
            if not items and prefix in name:
                code = collection.code + '_' + name
                item = MediaItem(collection=collection, code=code)
                item.title = name
                item.url = url
                item.public_access = 'full'
                item.save()
                print 'item created: ' + item.code
            else:
                print 'item already exists: ' + items[0].code



# -*- coding: utf-8 -*-
# Copyright (C) 2010 Samalyse SARL
# Copyright (C) 2010-2014 Parisson SARL

# This file is part of Telemeta.

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Authors: Olivier Guilyardi <olivier@samalyse.com>
#          David LIPSZYC <davidlipszyc@gmail.com>
#          Guillaume Pellerin <yomguy@parisson.com>

from __future__ import division
from django.utils.translation import ugettext_lazy as _
from telemeta.models.core import *
from telemeta.models.resource import *
from telemeta.models.collection import *


class MediaCorpus(MediaBaseResource):
    "Describe a corpus"

    element_type = 'corpus'
    children_type = 'collections'

    children = models.ManyToManyField(MediaCollection, related_name="corpus",
                                      verbose_name=_('collections'),  blank=True, null=True)
    recorded_from_year    = IntegerField(_('recording year (from)'), help_text=_('YYYY'))
    recorded_to_year      = IntegerField(_('recording year (until)'), help_text=_('YYYY'))

    objects = MediaCorpusManager()

    permissions = (("can_download_corpus_epub", "Can download corpus EPUB"),)

    @property
    def public_id(self):
        return self.code

    @property
    def has_mediafile(self):
        for child in self.children.all():
            if child.has_mediafile:
                return True
        return False

    def computed_duration(self):
        duration = Duration()
        for child in self.children.all():
            duration += child.computed_duration()
        return duration
    computed_duration.verbose_name = _('total available duration')

    class Meta(MetaCore):
        db_table = 'media_corpus'
        verbose_name = _('corpus')
        verbose_name_plural = _('corpus')
        ordering = ['code']


class MediaCorpusRelated(MediaRelated):
    "Corpus related media"

    resource = ForeignKey(MediaCorpus, related_name="related", verbose_name=_('corpus'))

    class Meta(MetaCore):
        db_table = 'media_corpus_related'
        verbose_name = _('corpus related media')
        verbose_name_plural = _('corpus related media')





# -*- coding: utf-8 -*-
# Copyright (C) 2007-2010 Samalyse SARL
# Copyright (C) 2010-2012 Parisson SARL

# This file is part of Telemeta.

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Authors: Olivier Guilyardi <olivier@samalyse.com>
#          Guillaume Pellerin <yomguy@parisson.com>

import re
import os
import sys
import csv
import time
import random
import datetime
import tempfile
import zipfile
import mimetypes
import json

from jsonrpc import jsonrpc_method

from django.utils.decorators import method_decorator
from django.contrib.auth import authenticate, login
from django.template import RequestContext, loader
from django import template
from django.http import HttpResponse, HttpResponseRedirect, StreamingHttpResponse
from django.http import Http404
from django.shortcuts import render_to_response, redirect, get_object_or_404
from django.views.generic import *
from django.conf import settings
from django.contrib import auth
from django.contrib import messages
from django.contrib.auth.decorators import login_required, permission_required
from django.core.context_processors import csrf
from django.forms.models import modelformset_factory, inlineformset_factory
from django.contrib.auth.models import User
from django.utils.translation import ugettext
from django.contrib.auth.forms import UserChangeForm
from django.core.exceptions import ObjectDoesNotExist
from django.contrib.syndication.views import Feed
from django.core.servers.basehttp import FileWrapper
from django.core.urlresolvers import reverse, reverse_lazy
from django.contrib.contenttypes.models import ContentType
from django.views.decorators.http import condition
from django.utils.translation import ugettext_lazy as _
from django.utils.translation import ugettext_lazy
from django.forms.models import model_to_dict
from django.views.generic.edit import DeletionMixin, BaseDeleteView
from django.contrib.sites.models import Site
from django.template.defaultfilters import slugify

from telemeta.models import *
import telemeta.models
import telemeta.interop.oai as oai
from telemeta.interop.oaidatasource import TelemetaOAIDataSource
from telemeta.util.unaccent import unaccent
from telemeta.util.unaccent import unaccent_icmp
from telemeta.util.logger import Logger
from telemeta.util.unicode import UnicodeCSVWriter, Echo
from telemeta.cache import TelemetaCache
import pages
from telemeta.forms import *
import jqchat.models

# Model type definition
mods = {'item': MediaItem, 'collection': MediaCollection,
        'corpus': MediaCorpus, 'fonds': MediaFonds, 'marker': MediaItemMarker, }


class TelemetaBaseMixin(object):

    MEDIA_ROOT = getattr(settings, 'MEDIA_ROOT')
    CACHE_DIR = os.path.join(MEDIA_ROOT, 'cache')
    cache_data = TelemetaCache(getattr(settings, 'TELEMETA_DATA_CACHE_DIR', CACHE_DIR))
    cache_export = TelemetaCache(getattr(settings, 'TELEMETA_EXPORT_CACHE_DIR', os.path.join(CACHE_DIR, 'export')))
    cache_tmp = TelemetaCache(getattr(settings, 'FILE_UPLOAD_TEMP_DIR', os.path.join(MEDIA_ROOT, 'tmp')))


class FixedFileWrapper(FileWrapper):
    def __iter__(self):
        self.filelike.seek(0)
        return self

def send_file(request, filename, content_type='image/jpeg'):
    """
    Send a file through Django without loading the whole file into
    memory at once. The FileWrapper will turn the file object into an
    iterator for chunks of 8KB.
    """
    wrapper = FixedFileWrapper(file(filename, 'rb'))
    response = StreamingHttpResponse(wrapper, content_type=content_type)
    response['Content-Length'] = os.path.getsize(filename)
    return response

def nginx_media_accel(request, filename):
    """Send a protected medie file through nginx with X-Accel-Redirect"""

    response = HttpResponse()
    url = settings.MEDIA_URL + filename
    # let nginx determine the correct content type
    response['Content-Type'] = ""
    response['X-Accel-Redirect'] = url
    return response

def render(request, template, data = None, mimetype = None):
    return render_to_response(template, data, context_instance=RequestContext(request),
                              mimetype=mimetype)

def stream_from_processor(decoder, encoder, flag):
    pipe = decoder | encoder
    for chunk in pipe.stream():
        yield chunk
    flag.value = True
    flag.save()

def stream_from_file(file):
    chunk_size = 0x100000
    f = open(file, 'r')
    while True:
        chunk = f.read(chunk_size)
        if not len(chunk):
            f.close()
            break
        yield chunk

def get_item_access(item, user):
    # Item access rules according to this workflow:
    # https://docs.google.com/spreadsheet/ccc?key=0ArKCjajoOT-fdDhJSDZoaUhqdDJvVkY5U3BXUWpNT0E#gid=0

    # Rolling publishing date : public access is automaticcaly given when time between recorded year
    # and current year is over the settings value PUBLIC_ACCESS_PERIOD and if item.auto_period_access == True

    if user.is_staff or user.is_superuser or user.has_perm('telemeta.can_play_all_items'):
        access = 'full'

    elif item.collection.public_access != 'mixed':
        if user.is_authenticated() :
            if item.collection.public_access == 'metadata' and item.collection.auto_period_access:
                access = 'full'
            else:
                access = item.collection.public_access
        else:
            access = item.collection.public_access

    elif item.collection.public_access == 'mixed':
        if user.is_authenticated() :
            if item.public_access == 'metadata' and item.auto_period_access:
                access = 'full'
            else:
                access = item.public_access
        else:
            access = item.public_access

    # Auto publish after a period given at settings.TELEMETA_PUBLIC_ACCESS_PERIOD
    if access != 'full' and access != 'none' and (item.auto_period_access or item.collection.auto_period_access):
        year_from = str(item.recorded_from_date).split('-')[0]
        year_to = str(item.recorded_to_date).split('-')[0]

        if year_from and not year_from == 0:
            year = year_from
        elif year_to and not year_to == 0:
            year = year_to
        else:
            year = 0

        if year and not year == 'None':
            year_now = datetime.datetime.now().strftime("%Y")
            if int(year_now) - int(year) >= settings.TELEMETA_PUBLIC_ACCESS_PERIOD:
                access = 'full'

    return access

def get_revisions(nb, user=None):
    last_revisions = Revision.objects.order_by('-time')
    if user:
        last_revisions = last_revisions.filter(user=user)
    last_revisions = last_revisions[0:nb]
    revisions = []

    for revision in last_revisions:
        for type in mods.keys():
            if revision.element_type == type:
                try:
                    element = mods[type].objects.get(pk=revision.element_id)
                except:
                    element = None
        if not element == None:
            revisions.append({'revision': revision, 'element': element})
    return revisions

def get_playlists(request, user=None):
    if not user:
        user = request.user
    playlists = []
    if user.is_authenticated():
        user_playlists = Playlist.objects.filter(author=user)
        for playlist in user_playlists:
            playlist_resources = PlaylistResource.objects.filter(playlist=playlist)
            resources = []
            for resource in playlist_resources:
                try:
                    for type in mods.keys():
                        if resource.resource_type == type:
                            element = mods[type].objects.get(id=resource.resource_id)
                except:
                    element = None
                resources.append({'element': element, 'type': resource.resource_type, 'public_id': resource.public_id })
            playlists.append({'playlist': playlist, 'resources': resources})
        #add by Killian Mary for sort playlist by title
        playlists.sort(key=lambda x: x['playlist'].title)
    return playlists


def get_playlists_names(request, user=None):
    if not user:
        user = request.user
    playlists = []
    if user.is_authenticated():
        user_playlists = user.playlists.all()
        for playlist in user_playlists:
            playlists.append({'playlist': playlist})
    return playlists


def check_related_media(medias):
    for media in medias:
        if not media.mime_type:
            media.set_mime_type()
            media.save()
        if not media.title and media.url:
            if 'https' in media.url:
                media.url = media.url.replace('https', 'http')
            import lxml.etree
            parser = lxml.etree.HTMLParser()
            tree = lxml.etree.parse(media.url, parser)
            try:
                title = tree.find(".//title").text
            except:
                title = media.url
            media.title = title.replace('\n', '').strip()
            media.save()

def auto_code(collection):
    items = collection.items.all()
    suffixes = []

    if items:
        for item in items:
            if '_' in item.public_id:
                try:
                    split = item.public_id.split('_')
                    suffix = int(split[-1])
                    prefix = split[:-1]
                except:
                    suffix = 999

                suffixes.append(suffix)

    if suffixes:
        return collection.code + '_' + str(max(suffixes)+1)
    else:
        return collection.code + '_001'


def get_room(content_type=None, id=None, name=None):
    rooms = jqchat.models.Room.objects.filter(content_type=content_type,
                                                object_id=id)
    if not rooms:
        room = jqchat.models.Room.objects.create(content_type=content_type,
                                          object_id=id,
                                          name=name[:254])
    else:
        room = rooms[0]
    return room


def get_kwargs_or_none(key, kwargs):
    if key in kwargs.keys():
        return kwargs[key]
    else:
        return None


def cleanup_path(path):
    new_path = []
    for dir in path.split(os.sep):
        new_path.append(slugify(dir))
    return os.sep.join(new_path)



import datetime
from django.conf import settings
from django.contrib.auth.models import User
from django.core.paginator import Paginator, InvalidPage
from django.http import Http404
from django.shortcuts import render_to_response, get_object_or_404
from django.template import RequestContext
from haystack.views import SearchView
from saved_searches.models import SavedSearch


SAVED_SEARCHES_PER_PAGE = getattr(settings, 'SAVED_SEARCHES_PER_PAGE', 50)
SAVED_SEARCHES_THRESHOLD = getattr(settings, 'SAVED_SEARCHES_THRESHOLD', 1)


class SavedSearchView(SearchView):
    """
    Automatically handles saving the queries when they are run.
    """
    search_key = 'general'

    def __init__(self, *args, **kwargs):
        if 'search_key' in kwargs:
            self.search_key = kwargs['search_key']
            del(kwargs['search_key'])

        super(SavedSearchView, self).__init__(*args, **kwargs)

    def save_search(self, page):
        """
        Only save the search if we're on the first page.
        This will prevent an excessive number of duplicates for what is
        essentially the same search.
        """
        if self.query and page.number == 1:
            # Save the search.
            saved_search = SavedSearch(
                search_key=self.search_key,
                user_query=self.query,
                result_count=len(self.results)
            )

            if hasattr(self.results, 'query'):
                query_seen = self.results.query.build_query()

                if isinstance(query_seen, basestring):
                    saved_search.full_query = query_seen

            if self.request.user.is_authenticated():
                saved_search.user = self.request.user

            saved_search.save()

    def create_response(self):
        """
        Saves the details of a user's search and then generates the actual
        ``HttpResponse`` to send back to the user.
        """
        (paginator, page) = self.build_page()
        self.save_search(page)

        context = {
            'query': self.query,
            'form': self.form,
            'page': page,
            'paginator': paginator,
        }
        context.update(self.extra_context())

        return render_to_response(self.template, context, context_instance=self.context_class(self.request))


def most_recent(request, username=None, search_key=None):
    """
    Shows the most recent search results.

    The ``username`` kwarg should be the ``username`` field of
    ``django.contrib.auth.models.User``. The ``search_key`` can be any string.

    Template::
        ``saved_searches/most_recent.html``
    Context::
        ``by_user``
            The ``User`` object corresponding to the username, if provided.
        ``by_search_key``
            The search_key, if provided.
        ``page``
            The request page of recent results (a dict of ``user_query`` + ``created``).
        ``paginator``
            The paginator object for the full result set.
    """
    if username is not None:
        user = get_object_or_404(User, username=username)
    else:
        user = None

    most_recent = SavedSearch.objects.most_recent(user=user, search_key=search_key, threshold=SAVED_SEARCHES_THRESHOLD)
    paginator = Paginator(most_recent, SAVED_SEARCHES_PER_PAGE)

    try:
        page = paginator.page(int(request.GET.get('page', 1)))
    except InvalidPage:
        raise Http404("Invalid page.")

    return render_to_response('saved_searches/most_recent.html', {
        'by_user': user,
        'by_search_key': search_key,
        'page': page,
        'paginator': paginator,
    }, context_instance=RequestContext(request))


def most_popular(request, username=None, search_key=None):
    """
    Shows the most popular search results.

    The ``username`` kwarg should be the ``username`` field of
    ``django.contrib.auth.models.User``. The ``search_key`` can be any string.

    Template::
        ``saved_searches/most_popular.html``
    Context::
        ``by_user``
            The ``User`` object corresponding to the username, if provided.
        ``by_search_key``
            The search_key, if provided.
        ``page``
            The request page of popular results (a dict of ``user_query`` + ``times_seen``).
        ``paginator``
            The paginator object for the full result set.
    """
    if username is not None:
        user = get_object_or_404(User, username=username)
    else:
        user = None

    most_recent = SavedSearch.objects.most_popular(user=user, search_key=search_key, threshold=SAVED_SEARCHES_THRESHOLD)
    paginator = Paginator(most_recent, SAVED_SEARCHES_PER_PAGE)

    try:
        page = paginator.page(int(request.GET.get('page', 1)))
    except InvalidPage:
        raise Http404("Invalid page.")

    return render_to_response('saved_searches/most_popular.html', {
        'by_user': user,
        'by_search_key': search_key,
        'page': page,
        'paginator': paginator,
    }, context_instance=RequestContext(request))




# -*- coding: utf-8 -*-

# Copyright (C) 2015 Angy Fils-Aimé, Killian Mary

# This file is part of Telemeta.

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


from haystack.views import *
from haystack.query import SearchQuerySet
from telemeta.models import *
from telemeta.forms.haystack_form import *
from saved_searches.views import SavedSearchView


class HaystackSearch(FacetedSearchView, SavedSearchView):

    search_key = 'quick'

    def __call__(self, request, type=None):
        self.type = type
        self.form_class = HaySearchForm
        self.selected_facet = self.selected_facet_list(request.GET.getlist('selected_facets', ['a']))
        print(self.selected_facet)
        if request.GET.get('results_page'):
            self.results_per_page = int(request.GET.get('results_page'))
        else:
            self.results_per_page = 20
        return super(HaystackSearch, self).__call__(request)

    def get_query(self):
        return super(HaystackSearch, self).get_query()

    def get_results(self):
        if(self.type == 'item'):
            return super(HaystackSearch, self).get_results().models(MediaItem)
        elif(self.type == 'corpus'):
            return super(HaystackSearch, self).get_results().models(MediaCorpus)
        elif(self.type == 'fonds'):
            return super(HaystackSearch, self).get_results().models(MediaFonds)
        else:
            return super(HaystackSearch, self).get_results().models(MediaCollection)

    def selected_facet_list(self, selected_facets):
        facet_list = []
        for facet in selected_facets:
            if ":" not in facet:
                continue

            field, value = facet.split(":", 1)

            if value and not value in facet_list:
                if field == 'digitized_exact':
                    facet_list.append('Sound')
                else:
                    facet_list.append(value)

        return facet_list

    def extra_context(self):
        extra = super(HaystackSearch, self).extra_context()
        extra['collection_count'] = super(HaystackSearch, self).get_results().models(MediaCollection).count()
        extra['item_count'] = super(HaystackSearch, self).get_results().models(MediaItem).count()
        extra['corpus_count'] = super(HaystackSearch, self).get_results().models(MediaCorpus).count()
        extra['fonds_count'] = super(HaystackSearch, self).get_results().models(MediaFonds).count()

        if extra['facets']:
            viewable_total = 0
            for viewable in extra['facets']['fields']['item_acces']:
                if viewable == 'none':
                    pass
                else:
                    viewable_total = viewable_total + viewable[1]

            extra['Published_count'] = self.get_results().narrow('item_status:Published').count()
            extra['Unpublished_count'] = self.get_results().narrow('item_status:Unpublished').count()
            extra['viewable_count'] = self.get_results().narrow('item_acces:full OR item_acces:mixed').narrow('digitized:T').count()
            extra['digitized_count'] = self.get_results().narrow('digitized:T').count()
            extra['CDR_count'] = self.get_results().narrow('physical_format:CDR').count()
            extra['Disque_count'] = self.get_results().narrow('physical_format:Disque').count()
            extra['Cylindre_count'] = self.get_results().narrow('physical_format:Cylindre').count()
            extra['Studio_count'] = self.get_results().narrow('recording_context:Studio').count()
            extra['Terrain_count'] = self.get_results().narrow('recording_context:Terrain').count()
            extra['Radio_count'] = self.get_results().narrow('recording_context:Radio').count()
            extra['Video_count'] = self.get_results().narrow('media_type:Video').count()
            extra['Audio_count'] = self.get_results().narrow('media_type:Audio').count()

        if self.type == 'item':
            extra['type'] = 'item'
        elif self.type == 'fonds':
            extra['type'] = 'fonds'
        elif self.type == 'corpus':
            extra['type'] = 'corpus'
        else:
            extra['type'] = 'collection'

        extra['selected_facets'] = self.selected_facet
        extra['selected_facets_url'] = self.request.GET.getlist('selected_facets')
        extra['results_page'] = self.results_per_page
        return extra

    #def auto_complete(request):
        #content = SearchQuerySet().autocomplete(content_auto=request.POST.get('seatch_text', ''))
        #return render_to_response('', {'content' : content])


class HaystackAdvanceSearch(SavedSearchView):

    search_key = 'advanced'

    def __call__(self, request, type=None):
        self.type = type
        if request.GET.get('results_page'):
            self.results_per_page = int(request.GET.get('results_page'))
        else:
            self.results_per_page = 20
        return super(HaystackAdvanceSearch, self).__call__(request)

    def get_query(self):
        #overwrite the get_query for begin search with any form
        if self.form.is_valid():
            return self.form.cleaned_data
        return ''

    def get_results(self):
        if(self.type == 'item'):
            return self.form.search().models(MediaItem)
        elif(self.type == 'fonds'):
            return self.form.search().models(MediaFonds)
        elif(self.type == 'corpus'):
            return self.form.search().models(MediaCorpus)
        else:
            return self.form.search().models(MediaCollection)

    def extra_context(self):
        extra = super(HaystackAdvanceSearch, self).extra_context()
        extra['fonds_count'] = self.form.search().models(MediaFonds).count()
        extra['corpus_count'] = self.form.search().models(MediaCorpus).count()
        extra['collection_count'] = self.form.search().models(MediaCollection).count()
        extra['item_count'] = self.form.search().models(MediaItem).count()

        if self.type == 'item':
            extra['type'] = 'item'
        elif  self.type == 'fonds':
            extra['type'] = 'fonds'
        elif(self.type == 'corpus'):
            extra['type'] = 'corpus'
        else:
            extra['type'] = 'collection'

        extra['results_page'] = self.results_per_page
        return extra




import os, sys
from telemeta.util.kdenlive.fade import AutoFade

if __name__ == '__main__':
    dir = sys.argv[-2]
    ext = sys.argv[-1]

    for filename in os.listdir(dir):
        prefix, extension = os.path.splitext(filename)
        path = dir + os.sep + filename
        flag = path + '.faded'
        if ext in extension and not os.path.exists(flag):
            os.system('cp ' + path + ' ' + path + '.bak')
            fade = AutoFade(path)
            data = fade.run()
            f = open(path, 'w')
            f.write(data)
            f.close()
            os.system('touch ' + flag)



# -*- coding: utf-8 -*-
# Copyright (C) 2010-2012 Parisson SARL

# This file is part of Telemeta.

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Authors: Guillaume Pellerin <yomguy@parisson.com>


from telemeta.views.core import *


class MarkerView(object):
    """Provide Collections web UI methods"""

    @jsonrpc_method('telemeta.add_marker')
    def add_marker(request, marker):
        # marker must be a dict
        if isinstance(marker, dict):
            item_id = marker['item_id']
            item = MediaItem.objects.get(id=item_id)
            m = MediaItemMarker(item=item)
            m.public_id = marker['public_id']
            m.time = float(marker['time'])
            m.title = marker['title']
            m.description = marker['description']
            m.author = User.objects.get(username=marker['author'])
            m.save()
            m.set_revision(request.user)
        else:
            raise 'Error : Bad marker dictionnary'

    @jsonrpc_method('telemeta.del_marker')
    def del_marker(request, public_id):
        m = MediaItemMarker.objects.get(public_id=public_id)
        m.delete()

    @jsonrpc_method('telemeta.get_markers')
    def get_markers(request, item_id):
        item = MediaItem.objects.get(id=item_id)
        markers = MediaItemMarker.objects.filter(item=item)
        list = []
        for marker in markers:
            dict = {}
            dict['public_id'] = marker.public_id
            dict['time'] = str(marker.time)
            dict['title'] = marker.title
            dict['description'] = marker.description
            dict['author'] = marker.author.username
            list.append(dict)
        return list

    @jsonrpc_method('telemeta.update_marker')
    def update_marker(request, marker):
        if isinstance(marker, dict):
            m = MediaItemMarker.objects.get(public_id=marker['public_id'])
            m.time = float(marker['time'])
            m.title = marker['title']
            m.description = marker['description']
            m.save()
            m.set_revision(request.user)
        else:
            raise 'Error : Bad marker dictionnary'

    @jsonrpc_method('telemeta.get_marker_id')
    def get_marker_id(request, public_id):
        marker = MediaItemMarker.objects.get(public_id=public_id)
        return marker.id





# -*- coding: utf-8 -*-
#
# Copyright (C) 2007 Samalyse SARL

# This file is part of Telemeta.

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Author: Olivier Guilyardi <olivier@samalyse.com>

import os
from xml.dom.minidom import getDOMImplementation, Node
import shutil
import md5
from django.conf import settings
from telemeta.models import MediaItem


class CollectionSerializer(object):
    """Provide backup-related features"""

    def __init__(self, collection, dest_dir='/tmp/'):
        self.collection = collection

    def __get_file_md5(self, path):
        "Compute the MD5 hash of a file (Python version of md5sum)"
        file = open(path, "rb")
        hash = md5.new()
        while True:
            buffer = file.read(0x10000)
            if len(buffer) == 0:
                break
            hash.update(buffer)
        file.close()
        return hash.hexdigest()

    def __get_media_filename(self, item):
        return str(item.id) + ".wav"

    def backup(self, dest_dir):
        self.coll_dir = dest_dir + "/" + str(self.collection.id)
        if not os.path.exists(self.coll_dir):
            os.makedirs(self.coll_dir)
        self.store_json()
        self.store_xml()
        self.store_files()

    def store_files(self):
        if self.collection.has_mediafile():
            md5_file = open(self.coll_dir + "/MD5SUM", "wb")

            items = self.collection.items.all()
            for item in items:
                if item.file:
                    dst_basename = self.__get_media_filename(item)
                    dst = self.coll_dir + "/" + dst_basename
                    shutil.copyfile(item.file.path, dst)
                    hash = self.__get_file_md5(dst)
                    md5_file.write(hash + "  " + dst_basename + "\n")

            md5_file.close()

    def store_json(self, dest_dir):
        """Serialize and store the collection with related items and media
        files into a subdirectory of the provided directory
        """
        json = self.collection.get_json()
        file = open(self.coll_dir + "/collection.json", "wb")
        file.write(json)
        file.close()
        self.store_files()

    def store_xml(self, dest_dir):
        """Serialize and store the collection with related items and media
        files into a subdirectory of the provided directory
        """
        xml = self.get_xml()
        file = open(self.coll_dir + "/collection.xml", "wb")
        file.write(xml.encode("utf-8"))
        file.close()
        self.store_files()

    def get_xml(self):
        """Return a string containing the XML representation of the collection
        and related items
        """
        impl = getDOMImplementation()
        doc = impl.createDocument(None, "telemeta", None)
        coll_doc = self.collection.to_dom()
        coll_node = doc.documentElement.appendChild(coll_doc.documentElement)
        coll_doc.unlink()
        items_node_name = MediaItem.get_dom_name() + "List"
        items_node = doc.createElement(items_node_name)
        coll_node.appendChild(items_node)

        items = self.collection.items.all()
        for item in items:
            if item.file:
                item.file = self.__get_media_filename(item)
            item_doc = item.to_dom()
            items_node.appendChild(item_doc.documentElement)
            item_doc.unlink()
        doc.normalize()

        return doc.toxml()





# -*- coding: utf-8 -*-
# Copyright (C) 2007-2010 Samalyse SARL
# Copyright (C) 2010-2012 Parisson SARL

# This file is part of Telemeta.

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Authors: Olivier Guilyardi <olivier@samalyse.com>
#          Guillaume Pellerin <yomguy@parisson.com>


from telemeta.views.core import *
from telemeta.views.epub import *
from django.utils.translation import ugettext_lazy as _


class ResourceView(object):
    """Provide Resource web UI methods"""

    types = {'corpus':
                {'model': MediaCorpus,
                'form' : MediaCorpusForm,
                'related': MediaCorpusRelated,
                'parent': MediaFonds,
                },
            'fonds':
                {'model': MediaFonds,
                'form' : MediaFondsForm,
                'related': MediaFondsRelated,
                'parent': None,
                }
            }

    def setup(self, type):
        self.model = self.types[type]['model']
        self.form = self.types[type]['form']
        self.related = self.types[type]['related']
        self.parent = self.types[type]['parent']
        self.type = type

    def detail(self, request, type, public_id, template='telemeta/resource_detail.html'):
        self.setup(type)
        resource = self.model.objects.get(code=public_id)
        children = resource.children.all()
        children = children.order_by('code')
        related_media = self.related.objects.filter(resource=resource)
        check_related_media(related_media)
        playlists = get_playlists(request)
        revisions = Revision.objects.filter(element_type=type, element_id=resource.id).order_by('-time')
        if revisions:
            last_revision = revisions[0]
        else:
            last_revision = None
        if self.parent:
            parents = self.parent.objects.filter(children=resource)
        else:
            parents = []

        return render(request, template, {'resource': resource, 'type': type, 'children': children,
                        'related_media': related_media, 'parents': parents, 'playlists': playlists,
                        'last_revision': last_revision })

    def edit(self, request, type, public_id, template='telemeta/resource_edit.html'):
        self.setup(type)
        resource = self.model.objects.get(code=public_id)
        if request.method == 'POST':
            form = self.form(data=request.POST, files=request.FILES, instance=resource)
            if form.is_valid():
                code = form.cleaned_data['code']
                if not code:
                    code = public_id
                form.save()
                resource.set_revision(request.user)
                return redirect('telemeta-resource-detail', self.type, code)
        else:
            form = self.form(instance=resource)
        return render(request, template, {'resource': resource, 'type': type, 'form': form,})

    def add(self, request, type, template='telemeta/resource_add.html'):
        self.setup(type)
        resource = self.model()
        if request.method == 'POST':
            form = self.form(data=request.POST, files=request.FILES, instance=resource)
            if form.is_valid():
                code = form.cleaned_data['code']
                if not code:
                    code = public_id
                form.save()
                resource.set_revision(request.user)
                return redirect('telemeta-resource-detail', self.type, code)
        else:
            form = self.form(instance=resource)
        return render(request, template, {'resource': resource, 'type': type, 'form': form,})

    def copy(self, request, type, public_id, template='telemeta/resource_edit.html'):
        self.setup(type)
        if request.method == 'POST':
            resource = self.model()
            form = self.form(data=request.POST, files=request.FILES, instance=resource)
            if form.is_valid():
                code = form.cleaned_data['code']
                if not code:
                    code = public_id
                resource.save()
                resource.set_revision(request.user)
                return redirect('telemeta-resource-detail', self.type, code)
        else:
            resource = self.model.objects.get(code=public_id)
            form = self.form(instance=resource)
        return render(request, template, {'resource': resource, 'type': type, "form": form,})

    def playlist(self, request, type, public_id, template, mimetype):
        self.setup(type)
        try:
            resource = self.model.objects.get(code=public_id)
        except ObjectDoesNotExist:
            raise Http404

        template = loader.get_template(template)
        context = RequestContext(request, {'resource': resource, 'host': request.META['HTTP_HOST']})
        return HttpResponse(template.render(context), content_type=mimetype)

    def delete(self, request, type, public_id):
        self.setup(type)
        resource = self.model.objects.get(code=public_id)
        revisions = Revision.objects.filter(element_type='resource', element_id=resource.id)
        for revision in revisions:
            revision.delete()
        resource.delete()
        return HttpResponseRedirect('/archives/'+self.type+'/')

    def related_stream(self, request, type, public_id, media_id):
        self.setup(type)
        resource = self.model.objects.get(code=public_id)
        media = self.related.objects.get(resource=resource, id=media_id)
        if media.file:
            response = StreamingHttpResponse(stream_from_file(media.file.path), content_type=media.mime_type)
        else:
            raise Http404
        return response

    def related_download(self, request, type, public_id, media_id):
        self.setup(type)
        resource = self.model.objects.get(code=public_id)
        media = self.related.objects.get(resource=resource, id=media_id)
        if media.file:
            filename = media.file.path.split(os.sep)[-1]
            response = StreamingHttpResponse(stream_from_file(media.file.path), content_type=media.mime_type)
            response['Content-Disposition'] = 'attachment; ' + 'filename=' + filename
        else:
            raise Http404
        return response


class ResourceMixin(View):

    types = {'corpus':
                {'model': MediaCorpus,
                'form' : MediaCorpusForm,
                'related': MediaCorpusRelated,
                'parent': MediaFonds,
                'inlines': [CorpusRelatedInline,]
                },
            'fonds':
                {'model': MediaFonds,
                'form' : MediaFondsForm,
                'related': MediaFondsRelated,
                'parent': None,
                'inlines': [FondsRelatedInline,]
                }
            }

    def setup(self, type):
        self.model = self.types[type]['model']
        self.form = self.types[type]['form']
        self.form_class = self.types[type]['form']
        self.related = self.types[type]['related']
        self.parent = self.types[type]['parent']
        self.inlines = self.types[type]['inlines']
        self.type = type

    def get_object(self):
        # super(CorpusDetailView, self).get_object()
        self.type = self.kwargs['type']
        self.setup(self.type)
        objs = self.model.objects.filter(code=self.kwargs['public_id'])
        if not objs:
            try:
                obj = self.model.objects.get(id=self.kwargs['public_id'])
            except:
                raise Http404
        else:
            obj = objs[0]
        return obj

    def get_context_data(self, **kwargs):
        context = super(ResourceMixin, self).get_context_data(**kwargs)
        context['type'] = self.type
        return context


class ResourceSingleMixin(ResourceMixin):

    def get_queryset(self):
        self.type = self.kwargs['type']
        self.setup(self.type)
        return self

    def get_context_data(self, **kwargs):
        context = super(ResourceMixin, self).get_context_data(**kwargs)
        resource = self.get_object()
        related_media = self.related.objects.filter(resource=resource)
        check_related_media(related_media)
        playlists = get_playlists_names(self.request)
        revisions = Revision.objects.filter(element_type=self.type, element_id=resource.pk).order_by('-time')
        context['resource'] = resource
        context['type'] = self.type
        context['related_media'] = related_media
        context['revisions'] = revisions
        if revisions:
            context['last_revision'] = revisions[0]
        else:
            context['last_revision'] = None
        if self.parent:
            context['parents'] = self.parent.objects.filter(children=resource)
        else:
            context['parents'] = []
        return context


class ResourceListView(ResourceMixin, ListView):

    template_name = "telemeta/resource_list.html"
    paginate_by = 20

    def get_queryset(self):
        self.type = self.kwargs['type']
        self.setup(self.type)
        return self.model.objects.all().order_by('code')

    def get_context_data(self, **kwargs):
        context = super(ResourceListView, self).get_context_data(**kwargs)
        context['count'] = self.object_list.count()
        return context


class ResourceDetailView(ResourceSingleMixin, DetailView):

    template_name = "telemeta/resource_detail.html"


class ResourceDetailDCView(ResourceDetailView):

    template_name = "telemeta/resource_detail_dc.html"


class ResourceAddView(ResourceMixin, CreateView):

    template_name = 'telemeta/resource_add.html'

    def get_queryset(self):
        self.type = self.kwargs['type']
        self.setup(self.type)
        return self

    def get_success_url(self):
        return reverse_lazy('telemeta-resource-list', kwargs={'type':self.kwargs['type']})

    @method_decorator(permission_required('telemeta.add_mediacorpus'))
    @method_decorator(permission_required('telemeta.add_mediafonds'))
    def dispatch(self, *args, **kwargs):
        return super(ResourceAddView, self).dispatch(*args, **kwargs)


class ResourceCopyView(ResourceSingleMixin, ResourceAddView):

    template_name = 'telemeta/resource_edit.html'

    def get_initial(self):
        return model_to_dict(self.get_object())

    def get_success_url(self):
        return reverse_lazy('telemeta-resource-list', kwargs={'type':self.kwargs['type']})
        # return reverse_lazy('telemeta-resource-detail', kwargs={'type':self.kwargs['type'], 'public_id':self.kwargs['public_id']})

    @method_decorator(permission_required('telemeta.add_mediacorpus'))
    @method_decorator(permission_required('telemeta.add_mediafonds'))
    def dispatch(self, *args, **kwargs):
        return super(ResourceCopyView, self).dispatch(*args, **kwargs)


class ResourceDeleteView(ResourceSingleMixin, DeleteView):

    template_name = 'telemeta/resource_confirm_delete.html'

    def get_success_url(self):
         return reverse_lazy('telemeta-resource-list', kwargs={'type':self.kwargs['type']})

    @method_decorator(permission_required('telemeta.delete_mediacorpus'))
    @method_decorator(permission_required('telemeta.delete_mediafonds'))
    def dispatch(self, *args, **kwargs):
        return super(ResourceDeleteView, self).dispatch(*args, **kwargs)


class ResourceEditView(ResourceSingleMixin, UpdateWithInlinesView):

    template_name = 'telemeta/resource_edit.html'

    def get_success_url(self):
        return reverse_lazy('telemeta-resource-detail', kwargs={'type':self.kwargs['type'], 'public_id':self.kwargs['public_id']})

    @method_decorator(permission_required('telemeta.change_mediacorpus'))
    @method_decorator(permission_required('telemeta.change_mediafonds'))
    def dispatch(self, *args, **kwargs):
        return super(ResourceEditView, self).dispatch(*args, **kwargs)


class ResourceEpubView(ResourceSingleMixin, BaseEpubMixin, View):
    "Download corpus data embedded in an EPUB3 file"

    def get(self, request, *args, **kwargs):
        self.setup_epub(self.get_object())
        if not os.path.exists(self.path):
            self.write_book()
        epub_file = open(self.path, 'rb')
        response = HttpResponse(epub_file.read(), content_type='application/epub+zip')
        response['Content-Disposition'] = "attachment; filename=%s" % self.filename + '.epub'
        return response


class ResourceEpubPasswordView(ResourceSingleMixin, FormView):

    template_name = 'telemeta/resource_epub_password.html'
    form_class = EpubPasswordForm

    def get_success_url(self):
        return reverse_lazy('telemeta-resource-epub-list', kwargs={'type': self.kwargs['type'], 'public_id': self.kwargs['public_id']})

    def form_valid(self, form):
        self.password = form.cleaned_data['password']
        if self.password != unicode('mélodie'.decode('utf-8')):
            messages.info(self.request, _("Bad password, please try again."))
            return redirect('telemeta-resource-password-epub', self.kwargs['type'], self.kwargs['public_id'])
        else:
            return redirect('telemeta-resource-epub-list', self.kwargs['type'], self.kwargs['public_id'])

        return super(ResourceEpubPasswordView, self).form_valid(form)

    def dispatch(self, *args, **kwargs):
        return super(ResourceEpubPasswordView, self).dispatch(*args, **kwargs)


class ResourceEpubListView(ResourceDetailView):

    template_name = 'telemeta/resource_epub_list.html'




#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Vérifier que les nouvelles cotes d'item :

- correspondent toutes à la collection décrite par le fichier .csv
  (le fichier .csv est nommé d'après la nouvelle cote de collection)

- sont uniques dans le fichiers .csv

- ont un des formats suivant :
    - soit CNRSMH_I_aaaa_nnn_mmm
    - soit CNRSMH_I_aaaa_nnn_mmm_tt
    - soit CNRSMH_I_aaaa_nnn_mmm_tt_pp
    - soit CNRSMH_E_aaaa_nnn_mmm_tt
    - soit CNRSMH_E_aaaa_nnn_mmm_tt_pp

- correspondent à fichier .wav (et qu'il n'y a pas de fichiers .wav
  supplémentaire)

Vérifier que le répertoire est nommé d'apprès la nouvelle cote de collection

Vérifier que la nouvelle cote de collection a l'un des formats suivant :
    - soit CNRSMH_I_aaaa_nnn
    - soit CNRSMH_E_aaaa_nnn_mmm

Vérifier que les fichiers .wav sont lisibles, ont une durée et sont identifés
comme WAV par audiolab.
"""


import os
import re
import sys
import csv
import xlrd
import datetime
import logging
import shutil

COLLECTION_OLD_PATTERN = [
        { 'format': 'BM.aaa.nnn.mmm',           'regex': r'^(BM)\.([0-9]{3})\.([0-9]{3})\.([0-9]{3})$'},
        { 'format': 'BM.aaaa.nnn.mmm/pp',       'regex': r'^(BM)\.([0-9]{4})\.([0-9]{3})\.([0-9]{3})/[0-9]{2}$'},
        { 'format': 'BM.aaaa.nnn.mmm',          'regex': r'^(BM)\.([0-9]{4})\.([0-9]{3})\.([0-9]{3})$'},
        { 'format': 'BM.aaaa.nnn.mmm/',         'regex': r'^(BM)\.([0-9]{4})\.([0-9]{3})\.([0-9]{3})/$'},
        { 'format': 'BM.aaaa.nnn.mmm/ppp',      'regex': r'^(BM)\.([0-9]{4})\.([0-9]{3})\.([0-9]{3})/[0-9]{3}$'},
        { 'format': 'BM.aaaa.nnn.mm/pp',        'regex': r'^(BM)\.([0-9]{4})\.([0-9]{3})\.([0-9]{2})/[0-9]{2}$'},
        { 'format': 'BM.aaaa.nnn',              'regex': r'^(BM)\.([0-9]{4})\.([0-9]{3})$'},
        { 'format': 'BM.aaa.nnn.mmm/pp',        'regex': r'^(BM)\.([0-9]{3})\.([0-9]{3})\.([0-9]{3})/[0-9]{2}$'},
        { 'format': 'BM.aaa.nnn FANTOME',       'regex': r'^(BM)\.([0-9]{3})\.([0-9]{3}) FANTOME$'},
        { 'format': 'BM.aaa.nnn',               'regex': r'^(BM)\.([0-9]{3})\.([0-9]{3})$'},
        { 'format': 'BM.aaa.nnnBISoo/pp',       'regex': r'^(BM)\.([0-9]{3})\.([0-9]{3})BIS([0-9]{2})/[0-9]{2}$'},
        { 'format': 'BM.aaa.nnn.mmm.ppp',       'regex': r'^(BM)\.([0-9]{3})\.([0-9]{3})\.([0-9]{3})\.[0-9]{3}$'},
        { 'format': 'BM.aaa.nnn.mmm/ppp',       'regex': r'^(BM)\.([0-9]{3})\.([0-9]{3})\.([0-9]{3})/[0-9]{3}$'},
        { 'format': 'BM.aaa.nnn/pp',            'regex': r'^(BM)\.([0-9]{3})\.([0-9]{3})/[0-9]{2}$'},
        { 'format': 'BM.aaa.nnn-BIS.ooo/pp',    'regex': r'^(BM)\.([0-9]{3})\.([0-9]{3})-BIS\.([0-9]{3})/[0-9]{2}$'},
        { 'format': 'BM.aaaa.nnn.mmm/NN',       'regex': r'^(BM)\.([0-9]{4})\.([0-9]{3})\.([0-9]{3})/NN$'},
        { 'format': 'BM.aaa.nnn.mmm/pp-DEPOT',  'regex': r'^(BM)\.([0-9]{3})\.([0-9]{3})\.([0-9]{3})/[0-9]{2}-DEPOT$'},
        { 'format': 'BM.aaa.nnn.mmm-o>p',       'regex': r'^(BM)\.([0-9]{3})\.([0-9]{3})\.([0-9]{3})-[0-9]>[0-9]$'},
        { 'format': 'CY.aaaa.nnn',              'regex': r'^(CY)\.([0-9]{4})\.([0-9]{3})$'},
        { 'format': 'DI.aaaa.nnn.mmm',          'regex': r'^(DI)\.([0-9]{4})\.([0-9]{3})\.([0-9]{3})$'},
        { 'format': 'DI.aaaa.nnn.mmm/pp',       'regex': r'^(DI)\.([0-9]{4})\.([0-9]{3})\.([0-9]{3})/[0-9]{2}$'},
        { 'format': 'DI.aaa.nnn.mmm',           'regex': r'^(DI)\.([0-9]{3})\.([0-9]{3})\.([0-9]{3})$'},
        { 'format': 'DI.aaa.nnn.mmm/pp',        'regex': r'^(DI)\.([0-9]{3})\.([0-9]{3})\.([0-9]{3})/[0-9]{2}$'},
        { 'format': 'DI.aaa.nnn.mmm-o/p',       'regex': r'^(DI)\.([0-9]{3})\.([0-9]{3})\.([0-9]{3})-[0-9]/[0-9]$'},
        { 'format': 'FANTOME 2*',               'regex': r'FANTOME 2\*$'},

        ## yomguy
        { 'format': 'BM.aaaa.nnn.mm',       'regex': r'^(BM)\.([0-9]{4})\.([0-9]{3})\.([0-9]{3})$'},
        #{ 'format': 'BM.aaaa.nnn.mmm/pp:ii-jj', 'regex': r'^(BM)\.([0-9]{4})\.([0-9]{3})\.([0-9]{3})/([0-9]{2})\:([0-9]{2})\-([0-9]{2})$'},
        #{ 'format': 'BM.aaaa.nnn.mmm/ppp:ii-jj', 'regex': r'^(BM)\.([0-9]{4})\.([0-9]{3})\.([0-9]{3})/([0-9]{2})\:([0-9]{2})\-([0-9]{2})$'},
        #{ 'format': 'BM.aaaa.nnn.mmm:ii-jj',    'regex': r'^(BM)\.([0-9]{4})\.([0-9]{3})\.([0-9]{3}):([0-9]{2})\-([0-9]{2})$'},
        ]

ITEM_NEW_PATTERN = [
        { 'format': 'CNRSMH_I_aaaa_nnn_mmm',           'regex': r'^(CNRSMH)_I_([0-9]{4})_([0-9]{3})_([0-9]{3})$'},
        { 'format': 'CNRSMH_I_aaaa_nnn_mmm_tt',        'regex': r'^(CNRSMH)_I_([0-9]{4})_([0-9]{3})_([0-9]{3})_([0-9]{2})$'},
        { 'format': 'CNRSMH_I_aaaa_nnn_mmm_tt_pp',     'regex': r'^(CNRSMH)_I_([0-9]{4})_([0-9]{3})_([0-9]{3})_([0-9]{2})_([0-9]{2})$'},
        { 'format': 'CNRSMH_E_aaaa_nnn_mmm_tt',        'regex': r'^(CNRSMH)_E_([0-9]{4})_([0-9]{3})_([0-9]{3})_([0-9]{2})$'},
        { 'format': 'CNRSMH_E_aaaa_nnn_mmm_tt_pp',     'regex': r'^(CNRSMH)_E_([0-9]{4})_([0-9]{3})_([0-9]{3})_([0-9]{2,3})_([0-9]{2})$'},

        # yomguy
        { 'format': 'CNRSMH_I_aaaa_nnn_mm',           'regex': r'^(CNRSMH)_I_([0-9]{4})_([0-9]{3})_([0-9]{2})$'},
        ]

COLLECTION_PATTERN = [
        { 'format': 'CNRSMH_I_aaaa_nnn',           'regex': r'^(CNRSMH)_I_([0-9]{4})_([0-9]{3})$'},
        { 'format': 'CNRSMH_E_aaaa_nnn_mmm',        'regex': r'^(CNRSMH)_E_([0-9]{4})_([0-9]{3})_([0-9]{3})$'},
        ]


def check_name(patterns, name):
    match = False
    for pattern in patterns:
        match = re.match(pattern['regex'], name)
        if match:
            break
    return match


class Logger:

    def __init__(self, file):
        self.logger = logging.getLogger('myapp')
        self.hdlr = logging.FileHandler(file)
        self.formatter = logging.Formatter('%(asctime)s %(levelname)s %(message)s')
        self.hdlr.setFormatter(self.formatter)
        self.logger.addHandler(self.hdlr)
        self.logger.setLevel(logging.INFO)

    def write_info(self, prefix, message):
        self.logger.info(' ' + prefix + ' : ' + message.decode('utf8'))

    def write_error(self, prefix, message):
        self.logger.error(prefix + ' : ' + message.decode('utf8'))


class CremCollection:

    def __init__(self, dir, logger):
        self.dir = dir
        self.dir_name = self.dir.split(os.sep)[-1]
        self.file_list = os.listdir(self.dir)
        self.logger = logger

    def xls_list(self):
        file_list = []
        for file in self.file_list:
            filename = os.path.basename(file)
            ext = os.path.splitext(file)[1]
            if not '.' == filename[0] and (ext == '.xls' or ext == '.XLS'):
                file_list.append(file)
        print file_list
        return file_list

    def wav_list(self):
        list = []
        for file in self.file_list:
            filename = os.path.basename(file)
            ext = os.path.splitext(file)[1]
            if not '.' == filename[0] and (ext == '.wav' or ext == '.WAV'):
                list.append(file)
            elif '.' == filename[0]:
                self.logger.write_error(file, 'Warning : fichier caché présent !')
        return list


class CremCSV:

    def __init__(self, file):
        self.csv_file = open(file, 'w')
        self.csv = csv.writer(self.csv_file,  delimiter=';')

    def close(self):
        self.csv_file.close()

class CremXLS:

    def __init__(self, file):
        self.first_row = 8
        self.original_col = 0
        self.new_col = 1
        self.book = xlrd.open_workbook(file)
        self.sheet = self.book.sheet_by_index(0)
        self.original_refs = self.original_refs()
        self.new_refs = self.new_refs()
        #print len(self.new_refs)
        while True:
            if len(self.original_refs) == 0 or len(self.new_refs) == 0:
                break
            else:
                if not 'CNRS' in self.new_refs[0].encode('utf8') \
                 and not  self.original_refs[0].encode('utf8') == '':
                    self.original_refs = self.original_refs[1:]
                    self.new_refs = self.new_refs[1:]
                else:
                    break

        self.size = max(len(self.new_refs), len(self.original_refs))

    def original_refs(self):
        col = self.sheet.col(self.original_col)
        list = []
        for cell in col[self.first_row:]:
            if cell.ctype == 1:
                list.append(cell.value)
        return list

    def new_refs(self):
        col = self.sheet.col(self.new_col)
        list = []
        for cell in col[self.first_row:]:
            if cell.ctype == 1:
                list.append(cell.value)
        return list


class CremItemFile:

    def __init__(self):
        self.media = ''

    def set_media(self, media):
        self.media = media

    def properties(self):
        self.frames = self.audio_file.get_nframes()
        self.samplerate = self.audio_file.get_samplerate()
        self.channels = self.audio_file.get_channels()
        self.format = self.audio_file.get_file_format()
        self.encoding = self.audio_file.get_encoding()


class CremCheck:

    def __init__(self, root_dir, log_file):
        self.root_dir = root_dir
        self.logger = Logger(log_file)
        dir_list = os.listdir(self.root_dir)
        list = []
        for dir in dir_list:
           if not dir[0] == '.':
               list.append(dir)
        self.dir_list = list

    def check_new_refs(self):
        for name in self.new_refs:
            return check_name(ITEM_PATTERN, name)

    def check(self):
        for dir in self.dir_list:
            collection = CremCollection(self.root_dir + dir, self.logger)
            msg = '************************ ' + collection.dir_name + ' ******************************'
            self.logger.write_info(collection.dir, msg[:70])

            xls_list = collection.xls_list()
            wav_list = collection.wav_list()

            if not check_name(COLLECTION_PATTERN, dir):
                self.logger.write_error(collection.dir, 'Le dossier de la collection est mal nommé -> SORTIE')
            elif len(xls_list) == 0:
                self.logger.write_error(collection.dir, 'PAS de fichier XLS dans le dossier collection -> SORTIE')
            elif len(xls_list) > 1:
                self.logger.write_error(collection.dir, 'Plusieurs fichiers XLS dans le dossier collection -> SORTIE')

            else:
                xls = CremXLS(self.root_dir + os.sep + dir + os.sep + xls_list[0])
                self.logger.write_info(collection.dir, 'XLS : ' + xls_list[0] + ' - Feuille : ' + xls.sheet.name.encode('utf8'))
                self.logger.write_info(collection.dir, 'Nombre d\'items détectés : ' + str(xls.size))
                csv_file = CremCSV(self.root_dir + dir + os.sep + collection.dir_name + '.csv')

                if len(wav_list) != xls.size:
                    self.logger.write_error(collection.dir, \
                    'Le nombre de références du fichier XLS (' + str(xls.size) + ') diffère du nombre de fichiers (' + str(len(wav_list)) + ')')

                temp_list = []
                item_file = CremItemFile()

                for i in range(0,xls.size):
                    error = False

                    try:
                        item_old = xls.original_refs[i]
                        #self.logger.write_error(collection.dir, item_old)
                    except:
                        item_old = ''
                        msg = 'Ligne ' + str(i+xls.first_row+1) + ' : l\'ancienne référence d\'item est inexistante'
                        self.logger.write_error(collection.dir, msg)
                        error = True
                        continue

                    try:
                        item = xls.new_refs[i]
                        #self.logger.write_error(collection.dir, item)
                    except:
                        item = ''
                        msg = 'Ligne ' + str(i+xls.first_row+1) + ' : la nouvelle référence d\'item est inexistante'
                        self.logger.write_error(collection.dir, msg)
                        error = True
                        continue

                    if not item in temp_list:
                        temp_list.append(item)
                    else:
                        msg =  'Ligne ' + str(i+xls.first_row+1) + ' : la référence d\'item ' + item.encode('utf8') + ' est multiple'
                        self.logger.write_error(collection.dir, msg)
                        error = True

                    #if not check_name(ITEM_OLD_PATTERN, item_old):
                        #msg = 'Ligne ' + str(i+xls.first_row+1) + ' : l\'ancienne référence d\'item ' + item_old.encode('utf8') + ' est mal formatée'
                        #self.logger.write_error(collection.dir, msg)

                    if not check_name(ITEM_NEW_PATTERN, item):
                        msg = 'Ligne ' + str(i+xls.first_row+1) + ' : la nouvelle référence d\'item ' + item.encode('utf8') + ' est mal formatée'
                        self.logger.write_error(collection.dir, msg)
                        error = True

                    if not collection.dir_name in item:
                        msg = 'Ligne ' + str(i+xls.first_row+1) + ' : la référence d\'item ' + item.encode('utf8') + ' ne correspond pas à celle de la collection'
                        self.logger.write_error(collection.dir, msg)
                        error = True

                    name_wav = item.encode('utf8') + '.wav'
                    if not name_wav in wav_list:
                        self.logger.write_error(collection.dir, 'Le fichier ' + item.encode('utf8') + '.wav n\'existe pas')
                    else:
                        item_file.set_media(collection.dir + os.sep + name_wav)
                        #if not item_file.is_wav():
                        #    self.logger.write_error(collection.dir, 'Le fichier ' + item.encode('utf8') + '.wav n\'est pas valide')
                        #    error = True

                    if not error:
                        csv_file.csv.writerow([xls.original_refs[i], xls.new_refs[i]])

                csv_file.close()

                for filename in wav_list:
                    if not check_name(ITEM_NEW_PATTERN, os.path.splitext(filename)[0]):
                        self.logger.write_error(collection.dir, 'Le nom du fichier ' + str(os.path.splitext(filename)[0]) + ' est mal formaté')

            msg = '********************************************************************************'
            self.logger.write_info(collection.dir, msg[:70])


def main():
    log_file = sys.argv[-1]
    root_dir = sys.argv[-2]
    log_tmp = log_file+'.tmp'

    c = CremCheck(root_dir, log_tmp)
    c.check()

    date = datetime.datetime.now().strftime("%x-%X").replace('/','_')
    shutil.copy(log_tmp,log_file+'-'+date+'.log')
    shutil.move(log_tmp,log_file)

if __name__ == '__main__':
    main()




import os, time

from optparse import make_option
from django.conf import settings
from django.core.management.base import BaseCommand, CommandError
from django.db import connections


class Command(BaseCommand):
    help = "wait for default DB connection"

    db_name = 'default'
    N = 10

    def handle(self, *args, **options):
        i = 0
        connected = False
        db_conn = connections[self.db_name]
        while not connected:
            try:
                c = db_conn.cursor()
                connected = True
            except:
                print('error connecting to DB...')
                if i > self.N:
                    print('...exiting')
                    raise
                print('...retrying')
                i += 1
                time.sleep(1)



# -*- coding: utf-8 -*-
# Copyright (C) 2007-2010 Samalyse SARL
# Copyright (C) 2010-2012 Parisson SARL

# This file is part of Telemeta.

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Authors: Olivier Guilyardi <olivier@samalyse.com>
#          Guillaume Pellerin <yomguy@parisson.com>


from telemeta.views.core import *


class InstrumentView(object):
    """Provide Instrument web UI methods"""

    @method_decorator(permission_required('telemeta.change_instrument'))
    def edit_instrument(self, request):

        instruments = Instrument.objects.all().order_by('name')
        if instruments == None:
            raise Http404
        return render(request, 'telemeta/instrument_edit.html', {'instruments': instruments})

    @method_decorator(permission_required('telemeta.add_instrument'))
    def add_to_instrument(self, request):

        if request.method == 'POST':
            instrument = Instrument(name=request.POST['value'])
            instrument.save()

        return self.edit_instrument(request)

    @method_decorator(permission_required('telemeta.change_instrument'))
    def update_instrument(self, request):

        if request.method == 'POST':
            Instrument.objects.filter(id__in=request.POST.getlist('sel')).delete()

        return self.edit_instrument(request)

    @method_decorator(permission_required('telemeta.change_instrument'))
    def edit_instrument_value(self, request, value_id):
        instrument = Instrument.objects.get(id__exact=value_id)
        instruments = Instrument.objects.all().order_by('name')

        content_type = ContentType.objects.get(app_label="telemeta", model='instrument')
        context = {}
        context['instrument'] = instrument
        context['instruments'] = instruments
        context['room'] = get_room(name=instrument._meta.verbose_name, content_type=content_type,
                                   id=instrument.id)

        return render(request, 'telemeta/instrument_edit_value.html', context)

    @method_decorator(permission_required('telemeta.change_instrument'))
    def update_instrument_value(self, request, value_id):

        if request.method == 'POST':
            instrument = Instrument.objects.get(id__exact=value_id)
            instrument.name = request.POST["value"]
            instrument.save()

        return self.edit_instrument(request)

    @method_decorator(permission_required('telemeta.change_instrument'))
    def replace_instrument_value(self, request, value_id):
        if request.method == 'POST':
            to_value_id = request.POST["value"]
            delete = False
            if 'delete' in request.POST.keys():
                delete = True

        obj_type = Instrument
        from_record = Instrument.objects.get(id__exact=value_id)
        to_record = Instrument.objects.get(id__exact=to_value_id)
        links = [rel.get_accessor_name() for rel in from_record._meta.get_all_related_objects()]

        for link in links:
            objects = getattr(from_record, link).all()
            for obj in objects:
                for name in obj._meta.get_all_field_names():
                    try:
                        field = obj._meta.get_field(name)
                        if field.rel.to == obj_type:
                            setattr(obj, name, to_record)
                            obj.save()
                    except:
                        continue

        if delete:
            from_record.delete()

        return self.edit_instrument(request)


class InstrumentAliasView(object):
    """Provide Instrument alias web UI methods"""

    @method_decorator(permission_required('telemeta.change_instrumentalias'))
    def edit_instrument(self, request):

        instruments = InstrumentAlias.objects.all().order_by('name')
        if instruments == None:
            raise Http404
        return render(request, 'telemeta/instrument_alias_edit.html', {'instruments': instruments})

    @method_decorator(permission_required('telemeta.add_instrument'))
    def add_to_instrument(self, request):

        if request.method == 'POST':
            instrument = InstrumentAlias(name=request.POST['value'])
            instrument.save()

        return self.edit_instrument(request)

    @method_decorator(permission_required('telemeta.change_instrumentalias'))
    def update_instrument(self, request):

        if request.method == 'POST':
            InstrumentAlias.objects.filter(id__in=request.POST.getlist('sel')).delete()

        return self.edit_instrument(request)

    @method_decorator(permission_required('telemeta.change_instrumentalias'))
    def edit_instrument_value(self, request, value_id):
        instrument = InstrumentAlias.objects.get(id__exact=value_id)
        instruments = InstrumentAlias.objects.all().order_by('name')

        content_type = ContentType.objects.get(app_label="telemeta", model='instrument')
        context = {}
        context['instrument'] = instrument
        context['instruments'] = instruments
        context['room'] = get_room(name=instrument._meta.verbose_name, content_type=content_type,
                                   id=instrument.id)

        return render(request, 'telemeta/instrument_alias_edit_value.html', context)

    @method_decorator(permission_required('telemeta.change_instrumentalias'))
    def update_instrument_value(self, request, value_id):

        if request.method == 'POST':
            instrument = InstrumentAlias.objects.get(id__exact=value_id)
            instrument.name = request.POST["value"]
            instrument.save()

        return self.edit_instrument(request)

    @method_decorator(permission_required('telemeta.change_instrumentalias'))
    def replace_instrument_value(self, request, value_id):
        if request.method == 'POST':
            to_value_id = request.POST["value"]
            delete = False
            if 'delete' in request.POST.keys():
                delete = True

        obj_type = InstrumentAlias
        from_record = InstrumentAlias.objects.get(id__exact=value_id)
        to_record = InstrumentAlias.objects.get(id__exact=to_value_id)
        links = [rel.get_accessor_name() for rel in from_record._meta.get_all_related_objects()]

        for link in links:
            objects = getattr(from_record, link).all()
            for obj in objects:
                for name in obj._meta.get_all_field_names():
                    try:
                        field = obj._meta.get_field(name)
                        if field.rel.to == obj_type:
                            setattr(obj, name, to_record)
                            obj.save()
                    except:
                        continue

        if delete:
            from_record.delete()

        return self.edit_instrument(request)
